/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.testng;

import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.ajax4jsf.bean.CalendarTestBean;
import org.ajax4jsf.template.Template;
import org.ajax4jsf.util.DateUtils;
import org.richfaces.SeleniumTestBase;
import org.testng.Assert;
import org.testng.annotations.Test;

import com.thoughtworks.selenium.SeleniumException;

public class CalendarTest extends SeleniumTestBase {

    static final String RESET_METHOD = "#{calendarBean.reset}";
    static final String FORM_ID = "_form:";

    static final String CONTROLS_FORM_ID = "_controls:";

    static final String availableDayCellClass = "rich-calendar-cell-size rich-calendar-cell rich-calendar-btn";
    
    static final String disabledDayCellClass = "rich-calendar-cell-size rich-calendar-cell rich-calendar-boundary-dates";

    String calendarId;

    String calendarHeaderId;
    
    String calendarFooterId;

    String ajaxSubmitId;

    String serverSubmitId;

    String statusId;

    String resetActionId;

    String testClientModeId;

    String setupActionId;

    String dateSelectionXpath;

    String timeSelectionXpath;

    String timeSelectionXpathMinusDiv;

    String timeHoursSelectionId;
    
    String timeMinutesSelectionId;
    
    String timeSelectionOkButtonId;
    
    String timeSelectionCancelButtonId;

    String selectedDateId;

    String currentDateId;

    String isPopupId;

    String datePatternId;

    String timeZoneId;

    String localeId;

    String currentDateHeaderXpath;

    String popupButtonId;

    String inputDateId;

    void initIds(String parentId) {
        calendarId = parentId + FORM_ID + "calendar";
        calendarHeaderId = calendarId + "Header";
        calendarFooterId = calendarId + "Footer";
        dateSelectionXpath = "//td[@id='"+calendarFooterId+"']/table/tbody/tr/td[1]";
        timeSelectionXpathMinusDiv = "//td[@id='"+calendarFooterId+"']/table/tbody/tr/td[3]";
        timeSelectionXpath = "//td[@id='"+calendarFooterId+"']/table/tbody/tr/td[3]/div";
        ajaxSubmitId = parentId + FORM_ID + "ajaxSubmit";
        serverSubmitId = parentId + FORM_ID + "serverSubmit";
        statusId = parentId + FORM_ID + "status";
        resetActionId = parentId + CONTROLS_FORM_ID + "resetAction";
        testClientModeId = parentId + CONTROLS_FORM_ID + "testClientMode";
        setupActionId = parentId + CONTROLS_FORM_ID + "setup";
        timeHoursSelectionId = calendarId + "TimeHours";
        timeMinutesSelectionId = calendarId + "TimeMinutes";
        timeSelectionOkButtonId = calendarId + "TimeEditorButtonOk";
        timeSelectionCancelButtonId = calendarId + "TimeEditorButtonCancel";
        selectedDateId = parentId + CONTROLS_FORM_ID + "selectedDate";
        currentDateId = parentId + CONTROLS_FORM_ID + "currentDate";
        isPopupId  = parentId + CONTROLS_FORM_ID + "isPopup";
        datePatternId = parentId + CONTROLS_FORM_ID + "datePattern";
        timeZoneId = parentId + CONTROLS_FORM_ID + "timeZone";
        localeId = parentId + CONTROLS_FORM_ID + "locale";
        currentDateHeaderXpath = "//td[@id='"+calendarHeaderId+"']/table/tbody/tr/td[3]/div";
        popupButtonId = calendarId + "PopupButton";
        inputDateId = calendarId + "InputDate";
    }

    String getStatus() {
        return getTextById(statusId);
    }

    void assertListeners(String... listener) {
        String status = getStatus();
        String s = status;
        String sum = "";
        for (String l : listener) {
            if (status.indexOf(l) == -1) {
                Assert.fail(l + " has been skipped");
            } else {
                s = s.replace(l, "");
            }
            sum += l;
        }
        if (s.length() > 0) {
            Assert.fail("The following listener were called but shouldn't: " + s);
        }
        if (!status.equals(sum)) {
            Assert.fail("Order of listeners call is incorrect. Should be: " + sum + ". But was : " + status);
        }
    }

    void changeDate() {
        String weekNumId = calendarId + "WeekNum2";
        selenium.click("//tr[@id='" + weekNumId + "']/td[starts-with(@class, '" + availableDayCellClass + "')]");
    }
    
    void changeCurrentDate(boolean wait4ajax) {
        selenium.click("//td[@id='" + calendarHeaderId + "']/table/tbody/tr/td/div");
        if (wait4ajax) {
            waitForAjaxCompletion();
        }
    }
    
    Calendar changeCurrentDate(Calendar c, boolean wait4ajax) {
        selenium.click("//td[@id='" + calendarHeaderId + "']/table/tbody/tr/td/div");
        if (wait4ajax) {
            waitForAjaxCompletion();
        }
        c.set(Calendar.YEAR, c.get(Calendar.YEAR) - 1);
        if (getCalendarDate().indexOf(c.get(Calendar.YEAR)) == -1) {
        	Assert.fail("Date is incorrect");
        }
        return c;
    }

    void reset() {
        clickCommandAndWait(resetActionId);
    }

    void switchToClientMode() {
        clickCommandAndWait(testClientModeId);
    }
    
    void showTimeSelectionWindow() {
    	selenium.click(timeSelectionXpath);
    }
    
    String getCalendarDate() {
    	return selenium.getText(currentDateHeaderXpath);
    }
    
    Calendar nextMonth(Calendar c) {
    	String date = getCalendarDate();
    	selenium.click("//td[@id='" + calendarHeaderId + "']/table/tbody/tr/td[@class='rich-calendar-tool'][3]/div");
    	Assert.assertFalse(getCalendarDate().equals(date), "Current date has not been changed after next month clicked");
    	c.set(Calendar.MONTH, c.get(Calendar.MONTH) + 1);
    	return c;
    }
    
    Calendar previousMonth(Calendar c) {
    	String date = getCalendarDate();
    	selenium.click("//td[@id='" + calendarHeaderId + "']/table/tbody/tr/td[@class='rich-calendar-tool'][2]/div");
    	Assert.assertFalse(getCalendarDate().equals(date), "Current date has not been changed after previous month clicked");
    	c.set(Calendar.MONTH, c.get(Calendar.MONTH) - 1);
    	return c;
    }
    
    void _testModelDataAppliedToClient(Calendar c) {
    	 String currentDate = selenium.getText(dateSelectionXpath);
         selenium.click("//table[@id='"+calendarId+"']/tbody/tr/td[text() = '13']");
         
         if (!selenium.getText(dateSelectionXpath).equals(currentDate)) {
         	Assert.fail("Enabled attribute of data model was not applied for client data. 13th day should disabled and should force date change after click.");
         }
         
         int currentMonth = c.get(Calendar.MONTH);
         
         String cellDay7StyleClass = selenium.getAttribute("//table[@id='"+calendarId+"']/tbody/tr/td[text() = '7']/@class");
         String cellDay7ModelClass = "styleClass"+currentMonth+"7";
         if (cellDay7StyleClass == null || cellDay7StyleClass.indexOf(cellDay7ModelClass) == -1) {
         	Assert.fail("Style class was not applied from data model to cell days. Style class for 7th day should contain [" + cellDay7ModelClass + "]. But has only + [" + cellDay7StyleClass +"]");
         }
         
         String cellDay5StyleClass = selenium.getAttribute("//table[@id='"+calendarId+"']/tbody/tr/td[text() = '5']/@class");
         String cellDay5ModelClass = "styleClass"+currentMonth+"5";
         if (cellDay5StyleClass == null || cellDay5StyleClass.indexOf(cellDay5ModelClass) == -1) {
         	Assert.fail("Style class was not applied from data model to cell days. Style class for 5th day should contain [" + cellDay5ModelClass + "]. But has only + [" + cellDay5StyleClass +"]");
         }
    }
    
    @Test
    public void testClientMode(Template template) {
    	renderPage(template, RESET_METHOD);
        initIds(getParentId());
        switchToClientMode();
        
        Calendar c = Calendar.getInstance();
        c = previousMonth(c);
		Assert.assertTrue(runScript("window.done").equals(Boolean.FALSE.toString().toLowerCase()), "Change of current date should not force ajax request in client mode.");
        
        try {
			_testModelDataAppliedToClient(c);
			c = nextMonth(c);
			_testModelDataAppliedToClient(c);
			c = nextMonth(c);
			_testModelDataAppliedToClient(c);
		} catch (AssertionError e) {
			Assert.fail("Calendar does not preload data in client mode for interval defined by 'preloadDateRangeBegin' & 'preloadDateRangeEnd'. It should preload data for last, current & next month. " + e);
		}
        
		boolean error = false;
		c = nextMonth(c);
		try {
			_testModelDataAppliedToClient(c);
		}catch (AssertionError e) {
			error = true;
		}
		
		if (!error) {
			Assert.fail("Calendar pre-loaded data for interval that not defined by 'preloadDateRangeBegin' & 'preloadDateRangeEnd'");
		}
		
    }
    
    @Test
    public void testDataModelAttribute(Template template) {
    	renderPage(template, RESET_METHOD);
        initIds(getParentId());
        
        Calendar c = Calendar.getInstance();
        
        _testModelDataAppliedToClient(c);
        
        try {
        	c = changeCurrentDate(c, true);
        }catch (SeleniumException exception) {
			Assert.fail("Celendar in ajax mode does not request for the next portion of data after current date has been changed");
		}
        
        _testModelDataAppliedToClient(c);
      
    }
    
    @Test
    public void testTimeSelection(Template template) {
    	renderPage(template, RESET_METHOD);
        initIds(getParentId());
       
        
        // Test time selection appearing
        showTimeSelectionWindow();
        AssertVisible(calendarId + "TimeEditorLayout", "Time selection dialog was not opened");
        AssertVisible(timeHoursSelectionId, "'Hour' input was not displayed");
        AssertVisible(timeMinutesSelectionId, "'Minutes' input was not displayed");
        AssertVisible(timeSelectionOkButtonId, "Time selection 'OK' button was not displayed");
        AssertVisible(timeSelectionCancelButtonId, "Time selection 'Cancel' button was not displayed");
        
        // Test time selection applying
        setValueById(timeHoursSelectionId, "10");
        setValueById(timeMinutesSelectionId, "00");
        clickById(timeSelectionOkButtonId);
        AssertNotVisible(calendarId + "TimeEditorLayout", "Time selection dialog was not hidden after 'Ok' button clicked");
        
        String time = selenium.getText(timeSelectionXpath);
        if (!"10:00".equals(time)) {
        	Assert.fail("Time selected has not been applied. Expected [10:00]. But was + [" + time+ "]");
        }
        
        // Test time cancel button 
        showTimeSelectionWindow();
        
        setValueById(timeHoursSelectionId, "18");
        setValueById(timeMinutesSelectionId, "50");
        clickById(timeSelectionCancelButtonId);
        AssertNotVisible(calendarId + "TimeEditorLayout", "Time selection dialog was not hidden after 'Cancel' button clicked");
        
        time = selenium.getText(timeSelectionXpath);
        if (!"10:00".equals(time)) {
        	Assert.fail("Time applied after cancel button clicked. Expected [10:00]. But was + [" + time+ "]");
        }
       
        // Test time applying in server side
    
        clickCommandAndWait(serverSubmitId);
        time = selenium.getText(timeSelectionXpath);
        if (!"10:00".equals(time)) {
        	Assert.fail("Time was not applied on server side. Expected [10:00]. But was + [" + time+ "]");
        }
        
    }

    @Test
    public void testListenersInAjaxMode(Template template) {
        renderPage(template, RESET_METHOD);
        initIds(getParentId());

        changeDate();

        clickAjaxCommandAndWait(ajaxSubmitId);
        assertListeners(CalendarTestBean.valueChangeListener);

        reset();

        changeCurrentDate(true);
        changeDate();
        clickAjaxCommandAndWait(ajaxSubmitId);
        assertListeners(CalendarTestBean.valueChangeListener, CalendarTestBean.currentDateChangeListener);

    }

    @Test
    public void testListenersInClientMode(Template template) {
        renderPage(template, RESET_METHOD);
        initIds(getParentId());
        switchToClientMode();

        changeDate();

        clickCommandAndWait(serverSubmitId);
        assertListeners(CalendarTestBean.valueChangeListener);

        reset();
        switchToClientMode();

        changeCurrentDate(false);
        changeDate();
        clickCommandAndWait(serverSubmitId);
        assertListeners(CalendarTestBean.valueChangeListener, CalendarTestBean.currentDateChangeListener);

    }

    //@Test
    public void testCalendarComponent(Template template) {
        renderPage(template);	

        String containerId = getParentId() + "_form:";
        String calendarOpenedId = containerId + "calendar";
        String calendarCollapsedId = calendarOpenedId + "Popup";
        String calendarInputDate = calendarOpenedId + "InputDate";
        String calendarPopupButton = calendarCollapsedId + "Button";
        String outputPanel = containerId + "outputPanel";

        Assert.assertFalse(isVisibleById(calendarOpenedId), "Calendar window should NOT be visible on the component!");

        writeStatus("Mouse click on calendar InputDate field");
        clickById(calendarInputDate);

        Assert.assertTrue(isVisibleById(calendarOpenedId), "Calendar window should be visible on the component!");

        writeStatus("Mouse click outside calendar");
        clickById(outputPanel);

        Assert.assertFalse(isVisibleById(calendarOpenedId), "Calendar window should NOT be visible on the component!");

        writeStatus("Mouse click on calendar popup button");
        clickById(calendarPopupButton);

        Assert.assertTrue(isVisibleById(calendarOpenedId), "Calendar window should be visible on the component!");
    }

    // @Test
    public void testSelectDateComponent(Template template) {
        renderPage(template);

        Date newSelectedDate = CalendarTestBean.getDayInMay(15);

        String containerId = getParentId() + "_form:";
        String calendarOpenedId = containerId + "calendar";
        String calendarCollapsedId = calendarOpenedId + "Popup";
        String calendarInputDate = calendarOpenedId + "InputDate";
        String calendarPopupButton = calendarCollapsedId + "Button";
        String outputPanel = containerId + "outputPanel";

        Assert.assertFalse(isVisibleById(calendarOpenedId), "Calendar window should NOT be visible on the component!");
        writeStatus("Mouse click on calendar popup button");
        clickById(calendarPopupButton);
        Assert.assertTrue(isVisibleById(calendarOpenedId), "Calendar window should be visible on the component!");

        String inputDateString = getValueById(calendarInputDate);
        Date readDate = null;
        try {
            readDate = CalendarTestBean.DATE_FORMAT.parse(inputDateString);
        } catch (ParseException parseException) {
            // skip exception
        }
        Assert.assertEquals(readDate, CalendarTestBean.DEFAULT_DATE, "Default date representation is wrong!");

        // click on 15th of May
        String newSelectedDateId = calendarOpenedId + "DayCell18";
        clickById(newSelectedDateId);

        writeStatus("Mouse click outside calendar");
        clickById(outputPanel);

        inputDateString = getValueById(calendarInputDate);
        try {
            readDate = CalendarTestBean.DATE_FORMAT.parse(inputDateString);
        } catch (ParseException parseException) {
            // skip exception
        }
        Assert.assertEquals(readDate, newSelectedDate, "Date representation after selecting 15.May.2008 is wrong!");

        Assert.assertFalse(isVisibleById(calendarOpenedId), "Calendar window should NOT be visible on the component!");
    }

    @Test
    public void testValueAndCurrentDateOfCalendarInCaseOfPopupTrue(Template template) {
        renderPage(template, RESET_METHOD);
        initIds(getParentId());
        String expectedSelectedDate = "03/03/2007 11:00";
        String expectedCurrentDate = "04/04/2008 13:00";
        setPopup(true);

        writeStatus("Check that in popup mode currentDate attribute is ignored in all cases");

        //1. value != null + curr_date != null
        setValueById(selectedDateId, expectedSelectedDate);
        setValueById(currentDateId, expectedCurrentDate);
        setup();

        showPopup();
        writeStatus("Check calendar popup is shown up");
        Assert.assertTrue(isVisible(calendarId), "Calendar popup is not visible");

        writeStatus("Check selected date");
        AssertValueEquals(inputDateId, expectedSelectedDate, "Calendar shows wrong date");

        writeStatus("Check current month and year. Remind! Current date has to be ignored. Value date is used instead");
        String currentDate = selenium.getText(currentDateHeaderXpath);
        Assert.assertEquals(currentDate, "March, 2007", "Calendar shows wrong current date");

        //2. value == null + curr_date != null
        setValueById(selectedDateId, "");
        setValueById(currentDateId, expectedCurrentDate);
        setup();

        showPopup();
        writeStatus("Check selected date is empty");
        AssertValueEquals(inputDateId, "", "Calendar value must be empty");

        writeStatus("Check current month and year. Remind! Current date has to be ignored as before. Value date is empty, present date is used");
        currentDate = selenium.getText(currentDateHeaderXpath);

        Locale locale = new Locale(selenium.getText(localeId));
        Date presentTime = Calendar.getInstance(locale).getTime();
        String month = DateUtils.month(presentTime, locale);
        int year = DateUtils.year(presentTime);
        String month_year = month + ", " + year;

        Assert.assertEquals(currentDate, month_year, "Calendar shows wrong current date");
    }

    @Test
    public void testValueAndCurrentDateOfCalendarWithPopupFalse(Template template) {
        renderPage(template, RESET_METHOD);
        initIds(getParentId());
        String expectedSelectedDate = "03/03/2007 11:00";
        String expectedCurrentDate = "04/04/2008 13:00";
        setPopup(false);

        //1. value != null + curr_date != null
        writeStatus("Check whether the component is present and up to the mark if value and currentDate are defined");
        setValueById(selectedDateId, expectedSelectedDate);
        setValueById(currentDateId, expectedCurrentDate);
        setup();

        writeStatus("Check calendar panel has been rendered");
        Assert.assertTrue(isVisible(calendarId), "Calendar panel is not visible");

        writeStatus("Check selected date");
        String date = selenium.getText(dateSelectionXpath);
        String time = selenium.getText(timeSelectionXpath);
        String date_time = date + " " + time;
        Assert.assertEquals(date_time, expectedSelectedDate, "Calendar shows wrong date");

        writeStatus("Check current month and year");
        String currentDate = selenium.getText(currentDateHeaderXpath);
        Assert.assertEquals(currentDate, "April, 2008", "Calendar shows wrong current date");

        //2. value != null + curr_date == null
        writeStatus("Check whether the component is present and up to the mark if value is given but currentDate is not");

        setValueById(selectedDateId, expectedSelectedDate);
        setValueById(currentDateId, "");
        setup();

        writeStatus("Check selected date");
        date = selenium.getText(dateSelectionXpath);
        time = selenium.getText(timeSelectionXpath);
        date_time = date + " " + time;
        Assert.assertEquals(date_time, expectedSelectedDate, "Calendar shows wrong date");

        writeStatus("Check current month and year. Current date is not specified. Value date will be used instead");
        currentDate = selenium.getText(currentDateHeaderXpath);
        Assert.assertEquals(currentDate, "March, 2007", "Calendar shows wrong current date");

        //3. value == null + curr_date == null
        writeStatus("Check whether the component is present and up to the mark if value and currentDate are not defined");
        setValueById(selectedDateId, "");
        setValueById(currentDateId, "");
        setup();

        writeStatus("Selected date is null. Selected value panel is not visible");
        Assert.assertFalse(isVisible(dateSelectionXpath), "Footer with selected date has to be invisible");
        Assert.assertFalse(isVisible(timeSelectionXpathMinusDiv), "Footer with selected date has to be invisible");

        writeStatus("Check current month and year. Current date and value are not defined. Present (do not mix with current time 8))");
        currentDate = selenium.getText(currentDateHeaderXpath);

        Locale locale = new Locale(selenium.getText(localeId));
        Date presentTime = Calendar.getInstance(locale).getTime();
        String month = DateUtils.month(presentTime, locale);
        int year = DateUtils.year(presentTime);
        String month_year = month + ", " + year;

        Assert.assertEquals(currentDate, month_year, "Calendar shows wrong current date");
    }

    @Test
    public void testDatePatternNonPopupMode(Template template) {
        renderPage(template, null);
        initIds(getParentId());
        String originalPattern = "MM/dd/yyyy HH:mm";
        writeStatus("Check 'datePattern' attribute in non-popup mode");

        String selectedDate = "03/03/2007 11:00";
        String expectedSelectedDate = "03/2007";
        String expectedPattern = "MM/yyyy";

        setValueById(selectedDateId, selectedDate);
        setValueById(currentDateId, "");
        setValueById(datePatternId, expectedPattern);
        setPopup(false);
        setup();

        try {
            writeStatus("Check displayed date. It should be in 'MM/yyyy' format - time part is not visible at all");
            String date = selenium.getText(dateSelectionXpath);
            Assert.assertEquals(date, expectedSelectedDate, "Calendar shows date in wrong format");
            Assert.assertFalse(isVisible(timeSelectionXpathMinusDiv), "Time part has to be invisible");
        } catch (AssertionError ae) {
            Assert.fail(ae.getMessage());
        } finally {
            setValueById(datePatternId, originalPattern);
            setup();
        }
    }

    @Test
    public void testDatePatternPopupMode(Template template) {
        renderPage(template, null);
        initIds(getParentId());
        String originalPattern = "MM/dd/yyyy HH:mm";
        writeStatus("Check 'datePattern' attribute in popup mode");

        String selectedDate = "03/03/2007 11:00";
        String expectedSelectedDate = "03/2007";
        String expectedPattern = "MM/yyyy";

        setValueById(selectedDateId, selectedDate);
        setValueById(currentDateId, "");
        setValueById(datePatternId, expectedPattern);

        setPopup(true);
        setup();
        showPopup();

        try {
            writeStatus("Check calendar popup is shown up and date in the input field is in proper format");
            Assert.assertTrue(isVisible(calendarId), "Calendar popup is not visible");
            AssertValueEquals(inputDateId, expectedSelectedDate, "Calendar input field shows wrong date");

            writeStatus("Check displayed date again. It as before should be in 'MM/yyyy' format - time part is not visible at all");
            String date = selenium.getText(dateSelectionXpath);
            Assert.assertEquals(date, expectedSelectedDate, "Calendar shows date in wrong format");
            Assert.assertFalse(isVisible(timeSelectionXpathMinusDiv), "Time part has to be invisible");
        } catch (AssertionError ae) {
            Assert.fail(ae.getMessage());
        } finally {
            setValueById(datePatternId, originalPattern);
            setup();
        }
    }

    private void setPopup(boolean isPopup) {
        runScript("$('" + isPopupId + "').checked=" + isPopup);
    }

    private void setup() {
        clickCommandAndWait(setupActionId);
    }

    private void showPopup() {
        writeStatus("Show popup");
        clickById(popupButtonId);
    }

    public String getTestUrl() {
        return "pages/calendar/calendarTest.xhtml";
    }
}
